==> file4life-1.1.5-source.tgz: file4life-1.1.5/src/F4L/Actions/Info.hs <==
{- This file is part of file4life, a file backup program written in Haskell.
 - file4life is Copyright (C) Antonio Bellezza 2014
 - file4life is free software: you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation, either version 3 of the License, or
 - (at your option) any later version.

 - file4life is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.

 - You should have received a copy of the GNU General Public License
 - along with file4life.  If not, see <http://www.gnu.org/licenses/>.
 -}
module F4L.Actions.Info (
    coverage
  , dumpFacts
  , dumpOps
  , fidsCoverage
  , listStorageContents
  , stats
  , unreachable
  ) where

import F4L
import F4L.Actions.Common
import F4L.Actions.Ops
import F4L.Actions.Storages( storagesForEnv )
import F4L.BufferMonad
import F4L.SeenTree as ST hiding ( removeFile )
import F4L.F4LMonad
import F4L.FSMonad hiding ( outputLineMsg, cleanLine )
import F4L.Knowledge
import F4L.Utils hiding ( fileSize )

import Control.Lens
import Control.Monad( liftM )
import qualified Data.ByteString.Lazy as BL
import Data.List( foldl', isPrefixOf, partition, sort )
import qualified Data.Map.Strict as M -- TODO: just use SM
import qualified Data.Map.Strict as SM
import qualified Data.Set as S



dumpOps :: ( FSMonad m, BufferMonad m, Knowledge k ) => BL.ByteString -> TopF4LMonad k m ()
dumpOps = mapM_ ( mapM_ $ outputMsg . show ) . map snd . readVOps


dumpFacts :: ( FSMonad m, BufferMonad m, Knowledge k ) => BL.ByteString -> TopF4LMonad k m ()
dumpFacts = loadOpsDo ( const $ return () ) ( outputMsg . show )


stats :: ( FSMonad m, BufferMonad m, Knowledge k, FactCounterClass k, FileSetClass k ) 
         => GenF4LMonad k s m [(String, String)]
stats = do
  count <- use $ st_knowledge . fact_counter
  files <- use $ st_knowledge . fs_files
  return
    [ ( "facts", hShow count )
    , ( "files", hShow $ S.size files )
    , ( "total size", hShow $ totalSize files )
    ]
    where
      totalSize = sum . map fid_length . S.toList


genCoverage :: ( FSMonad m, KnownDataClass k ) => ( SM.Map FileId FileData -> [ FileData ] ) -> GenF4LMonad k s m [(Int, Int, Float)]
genCoverage f = do
  dat <- use $ st_knowledge . kd_files . to f
  let byCopies = foldl' ( \ a d -> SM.insertWith (+) (S.size $ d ^. fd_seen ) 1 a ) SM.empty dat
  let total = fromIntegral $ length dat
  return $ sort [(i, c, 100.0 * fromIntegral c / total) | (i, c) <- SM.toList byCopies]


coverage :: ( FSMonad m, KnownDataClass k ) => GenF4LMonad k s m [(Int, Int, Float)]
coverage = genCoverage SM.elems


fidsCoverage :: ( FSMonad m, KnownDataClass k ) => [FileId] -> GenF4LMonad k s m [(Int, Int, Float)]
fidsCoverage fids = genCoverage f
  where
    f m =  map ( flip ( SM.findWithDefault noFileData ) m ) fids


unreachable :: ( FSMonad m, KnownDataClass k ) => F4LMonad k m ([FileId], [StorageId])
unreachable = do
  reachableSids <- liftM ( S.fromList . map fst . filter ( not . null . snd ) . SM.toList ) storagesForEnv
  let f :: ([FileId], S.Set StorageId) -> (FileId, FileData) -> ([FileId], S.Set StorageId)
      f (fids, sids) (fid, dat) = case partition ( `S.member` reachableSids ) $ S.toList $ S.fromList $ map fst $ S.toList $ view fd_seen $ dat of
        (_ : _, _) -> (fids, sids)
        ([], s) -> (fid : fids, sids `S.union` S.fromList s)
  fdata <- use $ st_knowledge . kd_files . to SM.toList
  return $ mapSnd S.toList $ foldl' f ([], S.empty) fdata


-- | Get storage contents by initial part of storage id
listStorageContents :: ( FSMonad m, BufferMonad m ) => String -> TopF4LMonad KnownData m [([FilePath], FileId)]
listStorageContents psid = do
  onErrorDie $ fromDB loadOps
  fullData <- use st_knowledge
  let contents = view kd_contents fullData
  case M.toList $ M.filterWithKey (const . ( psid `isPrefixOf` )) contents of
    [ ( _sid, dat ) ] ->
      return $ allFiles dat
    [] ->
      error "No storage with this ID found"
    xs ->
      error $ "Multiple storages found: " ++ show ( map fst xs )


{-
debugState :: ( FSMonad m, MonadIO m, Show k ) => F4LState k -> F4LMonad k m ()
debugState st = do
  say Quiet $ " " ++ ( show $ length $ show st )
  liftIO $ threadDelay 100000
  return ()
-}

==> file4life-1.2.0-source.tgz: file4life-1.2.0/src/F4L/Actions/Info.hs <==
{- This file is part of file4life, a file backup program written in Haskell.
 - file4life is Copyright (C) Antonio Bellezza 2014-2017
 - file4life is free software: you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation, either version 3 of the License, or
 - (at your option) any later version.

 - file4life is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.

 - You should have received a copy of the GNU General Public License
 - along with file4life.  If not, see <http://www.gnu.org/licenses/>.
 -}
module F4L.Actions.Info (
    coverage
  , dumpFacts
  , dumpOps
  , fidsCoverage
  , listStorageContents
  , stats
  , unreachable
  ) where

import F4L
import F4L.Actions.Common
import F4L.Actions.Ops
import F4L.Actions.Storages( storagesForEnv )
import F4L.BufferMonad
import F4L.F4LMonad
import F4L.FidStore
import F4L.FileTree as FT
import F4L.FSMonad hiding ( outputLineMsg, cleanLine )
import F4L.Knowledge
import F4L.StoryTree as ST hiding ( removeFile )
import F4L.Utils hiding ( fileSize )

import Lens.Micro.Platform
import Control.Monad( liftM )
import qualified Data.ByteString.Lazy as BL
import qualified Data.IntMap.Strict as IM
import Data.List( foldl', isPrefixOf, partition, sort )
import qualified Data.Map.Strict as M -- TODO: just use SM
import qualified Data.Map.Strict as SM
import Data.Maybe( fromMaybe )
import qualified Data.Set as S



dumpOps :: ( FSMonad m, BufferMonad m, Knowledge k ) => BL.ByteString -> TopF4LMonad k m ()
dumpOps = mapM_ ( ( mapM_ $ outputMsg . show ) . snd ) . readVOps


dumpFacts :: ( FSMonad m, BufferMonad m, Knowledge k ) => BL.ByteString -> TopF4LMonad k m ()
dumpFacts = loadOpsDo ( const $ return () ) ( outputMsg . show )


stats :: ( FSMonad m, BufferMonad m, Knowledge k, FactCounterClass k, FileSetClass k ) 
         => GenF4LMonad k s m [(String, String)]
stats = do
  count <- use $ st_knowledge . fact_counter
  files <- use $ st_knowledge . fs_files
  size <- use $ st_knowledge . fs_size
  return
    [ ( "facts", hShow count )
    , ( "files", hShow $ S.size files )
    , ( "total size", hShow size )
    ]
    where
      totalSize = sum . map fid_length . S.toList


genCoverage :: ( FSMonad m, KnownDataClass k )
  => ( IM.IntMap FileData -> [ FileData ] ) -> GenF4LMonad k s m [(Int, Int, Float)]
genCoverage f = do
  dat <- use $ st_knowledge . kd_files . to f
  let byCopies = foldl' ( \ a d -> SM.insertWith (+) ( S.size $ d ^. fd_seen ) 1 a ) SM.empty dat
  let total = fromIntegral $ length dat
  return $ sort [(i, c, 100.0 * fromIntegral c / total) | (i, c) <- SM.toList byCopies]


coverage :: ( FSMonad m, KnownDataClass k ) => GenF4LMonad k s m [(Int, Int, Float)]
coverage = genCoverage IM.elems


fidsCoverage :: ( FSMonad m, KnownDataClass k )
  => ( FileId -> Maybe FidIndex ) -> [ FileId ] -> GenF4LMonad k s m [(Int, Int, Float)]
fidsCoverage get_idx fids = genCoverage f
  where
    f m = map (
      fromMaybe noFileData . (>>= flip IM.lookup m ) . get_idx ) fids



unreachable :: ( FSMonad m, KnownDataClass k ) => F4LMonad k m ([FidIndex], [StorageId])
unreachable = do
  reachableSids <- liftM ( S.fromList . map fst . filter ( not . null . snd ) . SM.toList ) storagesForEnv
  let f :: ([FidIndex], S.Set StorageId) -> (FidIndex, FileData) -> ([FidIndex], S.Set StorageId)
      f (fids, sids) (fid, dat) = case partition ( `S.member` reachableSids ) $ S.toList $ S.fromList $ map fst $ S.toList $ view fd_seen $ dat of
        (_ : _, _) -> (fids, sids)
        ([], s) -> (fid : fids, sids `S.union` S.fromList s)
  fdata <- use $ st_knowledge . kd_files . to IM.toList
  return $ mapSnd S.toList $ foldl' f ([], S.empty) fdata


-- | Get storage contents by initial part of storage id
listStorageContents :: ( FSMonad m, BufferMonad m ) => String -> TopF4LMonad KnownData m [([FilePath], FidIndex)]
listStorageContents psid = do
  onErrorDie $ fromDB loadOps
  fullData <- use st_knowledge
  let contents = view kd_contents fullData
  case M.toList $ M.filterWithKey (const . ( psid `isPrefixOf` )) contents of
    [ ( _sid, dat ) ] ->
      return $ allFiles $ atTime endOfTime dat
    [] ->
      error "No storage with this ID found"
    xs ->
      error $ "Multiple storages found: " ++ show ( map fst xs )
